/*
 *  Converts CICYLIST from ASCII format to Mathematica format. The Euler characteristics are stored in a seperate file also in Mathematica format.
 *  (NOTE: LAST 4 BYTES OF OUTPUT FILE SHOULD BE CHANGE BY HAND ACCORDING TO: "}},}" --> "}}}")
 *
 *  Created by Alexander Haupt on 21/01/2010.
 *  Copyright 2013 Alexander Haupt. All rights reserved.
 *
 */

#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <time.h>
#include <libgen.h>

#if defined(__APPLE__) || defined(__MACH__) || defined(__OSX__)
  #include <malloc/malloc.h>
#else
  #include <malloc.h>
#endif

void proc_cicy_list(const char filename[]);
static inline void write_output1(FILE * fp, const int m, const int K, const register int n_r[], const register int q[]);


int main (int argc, const char * argv[]) {
	if (argc != 2) {
		printf("Usage: %s filename\n", argv[0]);
	}
	else {
		proc_cicy_list(argv[1]);
	}
	
	return 0;
}

void proc_cicy_list(const char filename[]) {
	int i, linecounter = 0, counter = 0;
	int m, K, eulernumber, orig_matno;
	int * n_r; int * prev_n_r = NULL;
	int * q;
	FILE * fp = fopen(filename, "r");
	
	char output_filename[80]; char output_en_filename[80]; char time_str[20]; time_t timer;
	timer = time(NULL);
	strftime(time_str, 50, "%y%m%d_%H%M%S", localtime(&timer));
	memset(output_filename, '\0', sizeof( output_filename )); memset(output_en_filename, '\0', sizeof( output_en_filename ));
	strncpy(output_filename, basename((char*)filename), strlen(basename((char*)filename)) - strlen(strrchr(basename((char*)filename), '.')));
	strcat(output_filename, ".m.txt");
	FILE * fp1 = fopen(output_filename, "a"); fprintf(fp1, "{");
	strncpy(output_en_filename, output_filename, strlen(output_filename) - 6);
	strcat(output_en_filename, "_en.m.txt");
	FILE * fp2 = fopen(output_en_filename, "a"); fprintf(fp2, "{");
	
	if (fp != NULL) {
		char line[128], strnum[2] = {0,0};
		
		while(fgets(line, sizeof line, fp) != NULL) { /* read a line */
			switch (linecounter) {
				case 0:
					if (line[0] == 'M') {
						sscanf(line, "MATRIX NUMBER :%6i\n", &orig_matno);
						linecounter++;
					}
					break;
				case 1:
					sscanf(line, "SIZE :%4i%3i\n", &m, &K);
					n_r = malloc(m*sizeof(int));
					q = malloc(m*K*sizeof(int));
					linecounter++;
					//printf("m=%i, K=%i, ", m, K);	// debug
					break;
				case 2:
					sscanf(line, "EULERNUMBER :%5i\n", &eulernumber);
					linecounter++;
					//printf("eulernumber=%i\n", eulernumber);	// debug
					break;
				default:
					if (linecounter <= (m+2)) {
						n_r[linecounter-3] = 0;
						for (i = 0; i < K; i++) {
							strnum[0] = line[(2*i)+4];
							q[i+((linecounter-3)*K)] = atoi(strnum);
							n_r[linecounter-3] += q[i+((linecounter-3)*K)];
						}
						n_r[linecounter-3]--;
						// only keep CICYs which are embedded in ambient spaces that don't contain P1s
						/*if ( (linecounter == 3) && (n_r[linecounter-3] == 1) ) {
							linecounter = 0;
							continue;
						}*/
						if (linecounter == (m+2)) {
							write_output1(fp1, m, K, n_r, q);
							fprintf(fp2, "%i,", eulernumber);
						}
						linecounter++;
					}
					else {
						free(n_r);
						free(q);
						linecounter = 0;
					}
					break;
			}
		}
		fclose(fp);
	}
	else {
		perror(filename); /* why didn't the file open? */
	}
	
	fprintf(fp1, "}");
	fclose(fp1);
	fprintf(fp2, "}");
	fclose(fp2);
}

static inline void write_output1(FILE * fp, const int m, const int K, const register int n_r[], const register int q[]) {
	int i, j;
	
	fprintf(fp, "{");
	for (i=0; i < m; i++) {
		fprintf(fp, "{");
		for (j=0; j < K; j++) {
			fprintf(fp, "%i", q[j+(i*K)]);
			if (j < (K-1)) {
				fprintf(fp, ",");
			}
		}
		fprintf(fp, "}");
		if (i < (m-1)) {
			fprintf(fp, ",");
		}		
	}
	fprintf(fp, "},");
}