/*
 *  Helper application for the Mathematica code "2ndfilter_1".
 *  If the spectrum of eigenvectors of qq^T is degenerate "2ndfilter_1" calls "equalq".
 *  It uses the "brute force" comparison of permutations to decide if two configurations are equivalent.
 *
 *  Created by Alexander Haupt on 11/12/2012.
 *  Copyright 2013 Alexander Haupt. All rights reserved.
 *
 */

#if defined(_WIN32) || defined(_WIN64)
  #error not tested on windows
#endif

#include <stdlib.h>
#include <stdio.h>
#include <stdbool.h>
#include <string.h>
#include <time.h>
#include <set>
#include <vector>

#if defined(__APPLE__) || defined(__MACH__) || defined(__OSX__)
  #include <malloc/malloc.h>
#else
  #include <malloc.h>
#endif

#include "is_already_there.h"

using namespace std;
using namespace __gnu_cxx;

int n;

void proc_cicy_list(const char filename[]);


int main (int argc, const char * argv[]) {
	if (argc != 3) {
		// input: n for CICY n-fold
		printf("Usage: %s n filename\n", argv[0]);
	}
	else {
		n = atoi(argv[1]);
		proc_cicy_list(argv[2]);
	}
	
	return 0;
}

void proc_cicy_list(const char filename[]) {
	int i, linecounter = 0, counter = 0;
	int m, K, eulernumber, orig_matno;
	int * n_r;
	int * q;		// q part of CICY config. matrix
	int num_P[2*(n+1)];	// array fr. 1 to (2*n+1). num_P[i] is the number of \CP^i factors in n_r. Note: n_r is ordered (ascendingly). num_P[0] is ignored
	
	FILE * fp = fopen(filename, "r");

	set<conf> conflist;
	set<conf>::iterator it;
	
	if (fp != NULL) {
		char line[128], strnum[2] = {0,0};
		
		while(fgets(line, sizeof line, fp) != NULL) { /* read a line */
			switch (linecounter) {
				case 0:
					if (line[0] == 'M') {
						sscanf(line, "MATRIX NUMBER :%6i\n", &orig_matno);
						linecounter++;
					}
					break;
				case 1:
					sscanf(line, "SIZE :%4i%3i\n", &m, &K);
					n_r = (int*)malloc(m*sizeof(int));
					q = (int*)malloc(m*K*sizeof(int));
					linecounter++;
					//printf("m=%i, K=%i, ", m, K);	// debug
					break;
				case 2:
					sscanf(line, "EULERNUMBER :%5i\n", &eulernumber);
					linecounter++;
					//printf("eulernumber=%i\n", eulernumber);	// debug
					break;
				default:
					if (linecounter <= (m+2)) {
						n_r[linecounter-3] = 0;
						for (i = 0; i < K; i++) {
							strnum[0] = line[(2*i)+4];
							q[i+((linecounter-3)*K)] = atoi(strnum);
							n_r[linecounter-3] += q[i+((linecounter-3)*K)];
						}
						n_r[linecounter-3]--;
						// only keep CICYs which are embedded in ambient spaces that don't contain P1s
						/*if ( (linecounter == 3) && (n_r[linecounter-3] == 1) ) {
							linecounter = 0;
							continue;
						}*/
						if (linecounter == (m+2)) {
							++counter;
							if (counter > 1) {
								if (is_already_there(&conflist, m, K, n_r, q) == conflist.end()) {
									add_conf(&conflist, m, K, n_r, q, eulernumber);
								}
							}
							else {
								add_conf(&conflist, m, K, n_r, q, eulernumber);
							}
						}
						linecounter++;
					}
					else {
						linecounter = 0;
					}
					break;
			}
		}
		fclose(fp);
	}
	else {
		perror(filename); /* why didn't the file open? */
	}

	// input counter != 2
	if (counter != 2) {printf("3\n"); fflush(stdout); return;}
	
	counter = conflist.size();	
	switch (counter) {
		case 1:
			printf("1\n"); fflush(stdout); // 1==true (the 2 configs are equivalent)
			break;
		case 2:
			printf("0\n"); fflush(stdout); // 0==false (the 2 configs are inequivalent)
			break;
		default:
			printf("3\n"); fflush(stdout); // 3==error value
			break;
	}
}