/*
 *  half_config.h
 *  Header file for half_config.cpp
 *
 *  Created by Alexander Haupt on 17/06/2011.
 *  Copyright 2013 Alexander Haupt. All rights reserved.
 *
 */

// Note : for C++ need to put definition {...} of inline function in header file if it's used in more than one .c/.cpp file!

#if defined(_WIN32) || defined(_WIN64)
  #error not tested on windows
#endif

#include <stdlib.h>
#include <stdio.h>
#include <stdbool.h>
#include <string.h>
#include <time.h>
#include <sys/time.h>
#include <math.h>
#include <set>

#if defined(__APPLE__) || defined(__MACH__) || defined(__OSX__)
  #include <malloc/malloc.h>
#else
  #include <malloc.h>
  #include <sys/resource.h>
#endif

#include "euler.h"

using namespace std;
using namespace __gnu_cxx;

struct qnode {
	int len;
	int * q;
};

void halfconfig_enumcicy(const int m, const int K, register int n_r[]);
bool fullK_comp(qnode a, qnode b);
bool K1_comp(qnode a, qnode b);
bool K2_comp(qnode a, qnode b);
void enumcicy(const int m, const int K, register int n_r[], const int q1[], const int q2[], set<qnode> * matrixlist);
void compute_indices_3fold(int indices[4], const int eulernumber, const int m, const int K, const int n_r[], const int q[]);

extern inline void init_val(const int dim, int array[], const int val) {
	int i = 0;
	
	for (i = 0; i < dim; i++) {
		//	for (i = dim; i--; ) {		// 15/03/2010: checked, but minimally slower than line above; also applies to for-loops elsewhere
		array[i] = val;
	}
	
	// tried the following: (found that above for-loop is ~2% faster!)
	//while(dim--)
	//  *array++ = val;
};

inline int * init_array(const int dim, const int val) {
	int * pointer;
	pointer = (int*)malloc(dim*sizeof(int));
	
	init_val(dim, pointer, val);
	return pointer;
};

static inline void set_smallest_q(const int m, const int K, const int n_r[], int q[]);

static inline void write_output(FILE * fp, const int counter, const int m, const int n_r[]);
static inline void write_output1(FILE * fp, const int counter, const int m, const int K, const register int n_r[], const register int q[], const int eulernumber);
static inline void write_output_CSV(FILE * fp, const int counter, const int m, const int K, const register int n_r[], const register int q[]);
void write_output2(const int len, const int q[]);
static inline void write_to_stdout(const int m, const int K, const int n_r[], const int q[]);
inline void debug_output(const int m, const int K, const int n_r[], const int q[]);

static inline int is_nontrivial(const int m, const int K, const int q[]);
int is_equal_q_matrix_abundancy(int len, int q1[], int q2[]);
inline int is_already_there(set<qnode> * matrixlist, int m, int K, int n_r[], int q[], int num_perms, int *n_r_perms[]);
int is_block_diag(const int m, const int K, const int q[]);
static inline int is_column_ordered(const int m, const int K, const register int q[]);
int is_0fold_id(const int m, const int K, const int n_r[], const int q[]);
int is_1fold_id(const int m, const int K, const int n_r[], const int q[]);
int is_2fold_id(const int m, const int K, const int n_r[], const int q[]);

static inline int * num_abundancy(int len, int q[]);
inline void add_qnode(set<qnode> * matrixlist, const int new_q[], const int len);

static inline int sum_q_column(const int a, const int m, const int K, const int q[]);

int increase_q(const int m, const int K, const int n_r[], register int q[]);
static inline int column_cmp(const int a, const int m, const int K, const register int q[]);

// memcpy optimised for int-arrays
extern inline void mymemcpy(int* pDest, const int* pSrc, int nSizeInWords) {
	while ( nSizeInWords-- )
		*pDest++ = *pSrc++;
};

static inline int mypow(const register int x, register int y);
static inline void next_idx1(const int m, const int K, register int idx[]);
static inline int is_nilpotent(const int m, const int K, const int n_r[], const int idx[]);

inline void init_num_P(const int m, const int n_r[], int num_P[]);
inline int calc_num_perms(const int num_P[], const int max_P);
int** init_permlist(const int m, const int n_r[], const int num_P[], const int num_perms);
void permutation(const int m, const int n_r[], const int num_P[], const int num_perms, int *n_r_perms[], int val);
void perm_rotate(int perm[], const int count);
inline void sort_q(const int m, const int K, int q[]);
int col_cmp(const void *a, const void *b);
inline void free_permlist(const int num_perms, int *n_r_perms[]);

// returns the complex dimension of the ambient space
static inline int dim(int m, int n_r[]) {
	int i, tmp = 0;
	
	for (i=0; i < m; i++) {
		tmp += n_r[i];
	}
	
	return tmp;
};