/*
 *  This program merges two CICY lists and removes redundancies due to row/column permutations.
 *  The input should be the output of "cat LIST1 LIST2 > MERGED_LIST".
 *  It uses the new streamlined recursive "is_already_there" function (as of 11/06/2012)
 *
 *  Created by Alexander Haupt on 11/06/2012.
 *  Copyright 2013 Alexander Haupt. All rights reserved.
 *
 */

#if defined(_WIN32) || defined(_WIN64)
  #error not tested on windows
#endif

#include <stdlib.h>
#include <stdio.h>
#include <stdbool.h>
#include <string.h>
#include <time.h>
#include <set>
#include <vector>
#include <sys/stat.h>
#include <errno.h>

#if defined(__APPLE__) || defined(__MACH__) || defined(__OSX__)
  #include <malloc/malloc.h>
#else
  #include <malloc.h>
#endif

#include "is_already_there.h"

using namespace std;
using namespace __gnu_cxx;

int n;

void proc_cicy_list(const int offset, const char filename[]);
static inline void write_output1(FILE * fp, const int counter, const int m, const int K, const register int n_r[], const register int q[], const int eulernumber);
inline void debug_output(const int m, const int K, const int n_r[], const int q[]);


int main (int argc, const char * argv[]) {
	if (argc != 4) {
		// input: "n" -- for CICY n-fold; "offset" -- the first offset matrices are not subjected to the "is_already_there" function.
		printf("Usage: %s n offset filename\n", argv[0]);
	}
	else {
		n = atoi(argv[1]);
		proc_cicy_list(atoi(argv[2]), argv[3]);
	}
	
	return 0;
}

void proc_cicy_list(const int offset, const char filename[]) {
	int i, linecounter = 0, counter = 0;
	int m, K, eulernumber, orig_matno;
	int * n_r;
	int * q;		// q part of CICY config. matrix
	int num_P[2*(n+1)];	// array fr. 1 to (2*n+1). num_P[i] is the number of \CP^i factors in n_r. Note: n_r is ordered (ascendingly). num_P[0] is ignored
	
	FILE * fp = fopen(filename, "r");

	set<conf> conflist;
	set<conf>::iterator it;
	
	char output_filename[50]; time_t timer;
	timer = time(NULL);
	sprintf(output_filename, "listjoin_%s", basename(filename));
	
	if (fp != NULL) {
		char line[128], strnum[2] = {0,0};
		
		while(fgets(line, sizeof line, fp) != NULL) { /* read a line */
			switch (linecounter) {
				case 0:
					if (line[0] == 'M') {
						sscanf(line, "MATRIX NUMBER :%6i\n", &orig_matno);
						linecounter++;
					}
					break;
				case 1:
					sscanf(line, "SIZE :%4i%3i\n", &m, &K);
					n_r = (int*)malloc(m*sizeof(int));
					q = (int*)malloc(m*K*sizeof(int));
					linecounter++;
					//printf("m=%i, K=%i, ", m, K);	// debug
					break;
				case 2:
					sscanf(line, "EULERNUMBER :%5i\n", &eulernumber);
					linecounter++;
					//printf("eulernumber=%i\n", eulernumber);	// debug
					break;
				default:
					if (linecounter <= (m+2)) {
						n_r[linecounter-3] = 0;
						for (i = 0; i < K; i++) {
							strnum[0] = line[(2*i)+4];
							q[i+((linecounter-3)*K)] = atoi(strnum);
							n_r[linecounter-3] += q[i+((linecounter-3)*K)];
						}
						n_r[linecounter-3]--;
						// only keep CICYs which are embedded in ambient spaces that don't contain P1s
						/*if ( (linecounter == 3) && (n_r[linecounter-3] == 1) ) {
							linecounter = 0;
							continue;
						}*/
						if (linecounter == (m+2)) {
							time(&timer); printf("%i:\t%s", ++counter, ctime(&timer)); fflush(stdout);
							if (counter > offset) {
								if (is_already_there(&conflist, m, K, n_r, q) == conflist.end()) {
									add_conf(&conflist, m, K, n_r, q, eulernumber);
								}
							}
							else {
								add_conf(&conflist, m, K, n_r, q, eulernumber);
							}
						}
						linecounter++;
					}
					else {
						linecounter = 0;
					}
					break;
			}
		}
		fclose(fp);
	}
	else {
		perror(filename); /* why didn't the file open? */
	}

	FILE * fp1 = fopen(output_filename, "a");
	counter = 0;
	for ( it=conflist.begin(); it != conflist.end(); it++ ) {
		write_output1(fp1, ++counter, (*it).m, (*it).K, (*it).n_r, (*it).q, (*it).eulernumber);
	}
	
	fclose(fp1);
	
	// checking that writing of output file was succesful otherwise write to stderr
	int fs_err = 0;
	struct stat st;
	if (stat(output_filename, &st) != 0) {fprintf(stderr, "stat() error, errno: %i\n", errno); fs_err = 1;}
	unsigned int size = (unsigned int)st.st_size;
	printf("Output file size: %u\n", size);
	if ( (size == 0) || (fs_err == 1) ) {
		counter = 0;
		for ( it=conflist.begin(); it != conflist.end(); it++ ) {
			write_output1(stderr, ++counter, (*it).m, (*it).K, (*it).n_r, (*it).q, (*it).eulernumber);
		}
	}
}

static inline void write_output1(FILE * fp, const int counter, const int m, const int K, const register int n_r[], const register int q[], const int eulernumber) {
	int i, j;
	
	fprintf(fp, "MATRIX NUMBER :%6i\nSIZE :%4i%3i\nEULERNUMBER :%5i\n", counter, m, K, eulernumber);
	for (i=0; i < m; i++) {
		fprintf(fp, "%i |", n_r[i]);
		for (j=0; j < K; j++) {
			fprintf(fp, " %i", q[j+(i*K)]);
		}
		fprintf(fp, "\n");
	}
	fprintf(fp, "\n");
}

inline void debug_output(const int m, const int K, const int n_r[], const int q[]) {
	int i, j;
	
	for (i = 0; i < m; i++) {
		printf("%i |", n_r[i]);
		for (j = 0; j < K; j++) {
			printf(" %i", q[j+(i*K)]);
		}
		printf("\n");
	}
	printf("\n");
}