/*
 *  This program reads a CICY list and outputs the same list but with the "MATRIX NUMBER" field in ascending 1, 2, 3, ... order (normalized sequence).
 *
 *  Created by Alexander Haupt on 27/02/2011.
 *  Copyright 2013 Alexander Haupt. All rights reserved.
 *
 */

#if defined(_WIN32) || defined(_WIN64)
  #error not tested on windows
#endif

#include <stdlib.h>
#include <stdio.h>
#include <stdbool.h>
#include <string.h>
#include <time.h>

#if defined(__APPLE__) || defined(__MACH__) || defined(__OSX__)
  #include <malloc/malloc.h>
#else
  #include <malloc.h>
#endif


void proc_cicy_list(const char filename[]);
static inline void write_output1(FILE * fp, const int counter, const int m, const int K, const register int n_r[], const register int q[], const int eulernumber);
inline void debug_output(const int m, const int K, const int n_r[], const int q[]);


int main (int argc, const char * argv[]) {
	if (argc != 2) {
		printf("Usage: %s filename\n", argv[0]);
	}
	else {
		proc_cicy_list(argv[1]);
	}
	
	return 0;
}

void proc_cicy_list(const char filename[]) {
	int i, linecounter = 0, counter = 0;
	int m, K, eulernumber, orig_matno;
	int * n_r;
	int * q;	// q part of CICY config. matrix

	FILE * fp = fopen(filename, "r");
	
	char output_filename[50]; char time_str[20]; time_t timer;
	timer = time(NULL);
	strftime(time_str, 50, "%y%m%d_%H%M%S", localtime(&timer));
	sprintf(output_filename, "matno_output_%s.txt", time_str);
	FILE * fp1 = fopen(output_filename, "a");	
	
	if (fp != NULL) {
		char line[128], strnum[2] = {0,0};
		
		while(fgets(line, sizeof line, fp) != NULL) { /* read a line */
			switch (linecounter) {
				case 0:
					if (line[0] == 'M') {
						sscanf(line, "MATRIX NUMBER :%6i\n", &orig_matno);
						linecounter++;
					}
					break;
				case 1:
					sscanf(line, "SIZE :%4i%3i\n", &m, &K);
					n_r = malloc(m*sizeof(int));
					q = malloc(m*K*sizeof(int));
					linecounter++;
					//printf("m=%i, K=%i, ", m, K);	// debug
					break;
				case 2:
					sscanf(line, "EULERNUMBER :%5i\n", &eulernumber);
					linecounter++;
					//printf("eulernumber=%i\n", eulernumber);	// debug
					break;
				default:
					if (linecounter <= (m+2)) {
						n_r[linecounter-3] = 0;
						for (i = 0; i < K; i++) {
							strnum[0] = line[(2*i)+4];
							q[i+((linecounter-3)*K)] = atoi(strnum);
							n_r[linecounter-3] += q[i+((linecounter-3)*K)];
						}
						n_r[linecounter-3]--;
						// only keep CICYs which are embedded in ambient spaces that don't contain P1s
						/*if ( (linecounter == 3) && (n_r[linecounter-3] == 1) ) {
							linecounter = 0;
							continue;
						}*/
						if (linecounter == (m+2)) {
							write_output1(fp1, ++counter, m, K, n_r, q, eulernumber);
						}
						linecounter++;
					}
					else {
						linecounter = 0;
					}
					break;
			}
		}
		fclose(fp);
	}
	else {
		perror(filename); /* why didn't the file open? */
	}

	fclose(fp1);
}

static inline void write_output1(FILE * fp, const int counter, const int m, const int K, const register int n_r[], const register int q[], const int eulernumber) {
	int i, j;
	
	fprintf(fp, "MATRIX NUMBER :%6i\nSIZE :%4i%3i\nEULERNUMBER :%5i\n", counter, m, K, eulernumber);
	for (i=0; i < m; i++) {
		fprintf(fp, "%i |", n_r[i]);
		for (j=0; j < K; j++) {
			fprintf(fp, " %i", q[j+(i*K)]);
		}
		fprintf(fp, "\n");
	}
	fprintf(fp, "\n");
}

inline void debug_output(const int m, const int K, const int n_r[], const int q[]) {
	int i, j;
	
	for (i = 0; i < m; i++) {
		printf("%i |", n_r[i]);
		for (j = 0; j < K; j++) {
			printf(" %i", q[j+(i*K)]);
		}
		printf("\n");
	}
	printf("\n");
}